unit AutorunPlugin;

interface

uses Windows;

type
  // Common function prototypes
  TPluginProc = function(Data: Pointer): integer; stdcall;
  TMacroProc = function(Data: PWideChar; dwSize: cardinal): integer; stdcall;
  TApiProc = function(FuncIndex: integer; Data: Pointer): integer; stdcall;

  // record for Init function
  PAutorunInterface = ^TAutorunInterface;
  TAutorunInterface = record
    dwSize:    DWORD;        // struct size
    hWnd:      HWND;         // TC window handle
    lpAdapter: Pointer;      // pointer to adapter function
  end;

  // parameters for plugin functions
  PAutorunFuncParams = ^TAutorunFuncParams;
  TAutorunFuncParams = record
    dwSize:    DWORD;        // struct size
    Param:     PWideChar;    // parameters string
    Value:     PWideChar;    // return value
    dwBufSize: DWORD;        // return value buffer size in WideChar's
    nError:    integer;      // user error value (non-fatal)
  end;

  // function definition
  PPluginFuncInfo = ^TPluginFuncInfo;
  TPluginFuncInfo = record
    dwSize:     DWORD;       // struct size
    dwFlags:    DWORD;       // function flags
    lpFuncName: PWideChar;   // name of function
    lpFuncAddr: Pointer;     // function address
  end;

  // variable definition
  PVarInfo = ^TVarInfo;
  TVarInfo = record
    dwSize:   DWORD;         // struct size
    dwFlags:  DWORD;         // variable flags
    lpName:   PWideChar;     // variable name
    lpData:   Pointer;       // variable value, buffer or function address
    nBufSize: DWORD;         // buffer size in BYTES
  end;

  // error reporting
  PErrorReport = ^TErrorReport;
  TErrorReport = record
    dwSize: DWORD;           // struct size
    nMsgID: integer;         // message ID
    lpText: PWideChar;       // message text / std param
    nFlags: integer;         // reserved for future use
  end;

  // common record for API functions
  PGetData = ^TGetData;
  TGetData = record
    lpText:     PWideChar;     // text/name/caption
    lpBuffer:   PWideChar;     // buffer
    dwBufSize: DWORD;         // buffer size in WideChar's
  end;

  // get value definition
  PValueInfo = ^TValueInfo;
  TValueInfo = record
    dwSize:      DWORD;         // struct size
    dwValueType: DWORD;         // value type
    dwFlags:     DWORD;         // optional flags
    lpName:      PWideChar;     // value name
    lpData:      Pointer;       // value
    nBufSize:    DWORD;         // buffer size in BYTES
  end;

  PExpandVars = ^TExpandVars;
  TExpandVars = TGetData;

  PParseNextData = ^TParseNextData;
  TParseNextData = TGetData;

  PCallCmd = ^TCallCmd;
  TCallCmd = TGetData;

const
  // common result
  AU_RESULT_OK = 0;
  AU_RESULT_UNKNOWN_ERROR = 1;
  AU_RESULT_INSUFFICIENT_BUFFER = 2;
  AU_RESULT_NOTENOUGHPARAMS = 3;
  AU_RESULT_WRONGPARAM = 4;

  // Function return values (error codes)
  AU_RESULT_VARNOEXIST = 2;
  AU_RESULT_NOREDEFINECONST = 3;

  // Parser function return values
  AU_PARSER_NOMOREPARAMS = 0;
  AU_PARSER_REGULARPARAM = 1;
  AU_PARSER_NAMEDPARARAM = 2;

  // Flags for set variables proc
  AU_VAR_CONSTANT = 1;
  AU_VAR_MACRO    = 2;

  // Flags for set functions proc
  AU_FUNC_RETURNVALUE = 1;
  AU_FUNC_BOOLEAN =     2;

  // Indexes of API functions
  AU_PROC_ADDFUNC     = 1;
  AU_PROC_PARSER_NEXT = 2;
  AU_PROC_GETVAR      = 3;
  AU_PROC_SETVAR      = 4;
  AU_PROC_EXPANDVARS  = 5;
  AU_PROC_REPORTERROR = 6;
  AU_PROC_CALLCOMMAND = 7;
  AU_PROC_GETVALUE    = 8;

  // Indexes of GetAutorunValue function
  AU_VALUE_TCCOMMAND  = 1;

function AddFunc(AName: PWideChar; AAddr: Pointer; AFlags: integer): integer;
function ParseNext(AParamStr: PWideChar; ABuffer: PWideChar; ABufSize: DWORD): integer;
function GetVar(AVarName: PWideChar; Buffer: PWideChar; nBufferSize: DWORD): integer;
function SetVar(const AName, AValue: PWideChar; AFlags: integer): integer;
function SetVarMacro(const AName: PWideChar; AFuncAddr: Pointer): integer;
function ExpandVars(const AText: PWideChar; ABuffer: PWideChar; ABufSize: DWORD): integer;
function ReportError(MsgID: integer; const AParam: PWideChar): integer; overload;
function ReportError(const AParam: PWideChar): integer; overload;
function CallCommand(const ACmdLine: PWideChar; AResult: PWideChar; AResultSize: DWORD): integer; overload;
function CallCommand(const ACmdLine: PWideChar): integer; overload;
function GetAutorunValue(AValueType: integer; const AValueName: PWideChar; ABuffer: Pointer; nBufSize, AFlags: DWORD): integer;

var
  Adapter: Pointer;

implementation

function AddFunc(AName: PWideChar; AAddr: Pointer; AFlags: integer): integer;
var
  FuncInfo: TPluginFuncInfo;
begin
  with FuncInfo do
  begin
    dwSize    := SizeOf(TPluginFuncInfo);
    dwFlags    := AFlags;
    lpFuncName := AName;
    lpFuncAddr := AAddr;
  end;
  Result := TAPIProc(Adapter)(AU_PROC_ADDFUNC, @FuncInfo);
end;

function ParseNext(AParamStr: PWideChar; ABuffer: PWideChar; ABufSize: DWORD): integer;
var
  Data: TParseNextData;
begin
  with Data do
  begin
    lpText    := AParamStr;
    lpBuffer  := ABuffer;
    dwBufSize := ABufSize;
  end;
  Result := TAPIProc(Adapter)(AU_PROC_PARSER_NEXT, @Data);
end;

function GetVar(AVarName: PWideChar; Buffer: PWideChar; nBufferSize: DWORD): integer;
var
  Data: TVarInfo;
begin
  with Data do
  begin
    dwSize    := SizeOf(TVarInfo);
    lpName   := AVarName;
    lpData  := Buffer;
    nBufSize := nBufferSize;
  end;
  Result := TAPIProc(Adapter)(AU_PROC_GETVAR, @Data);
end;

function SetVar(const AName, AValue: PWideChar; AFlags: integer): integer;
var
  VarInfo: TVarInfo;
begin
  with VarInfo do
  begin
    dwSize    := SizeOf(TVarInfo);
    dwFlags    := AFlags;
    lpName  := AName;
    lpData := AValue;
  end;
  Result := TAPIProc(Adapter)(AU_PROC_SETVAR, @VarInfo);
end;


function SetVarMacro(const AName: PWideChar; AFuncAddr: Pointer): integer;

var
  VarInfo: TVarInfo;
begin
  with VarInfo do
  begin
    dwSize  := SizeOf(TVarInfo);
    dwFlags := AU_VAR_MACRO;
    lpName  := AName;
    lpData  := AFuncAddr;
  end;
  Result := TAPIProc(Adapter)(AU_PROC_SETVAR, @VarInfo);
end;


function ExpandVars(const AText: PWideChar; ABuffer: PWideChar; ABufSize: DWORD): integer;
var
  Data: TExpandVars;
begin
  with Data do
  begin
    lpText     := AText;
    lpBuffer   := ABuffer;
    dwBufSize  := ABufSize;
  end;
  Result := TAPIProc(Adapter)(AU_PROC_EXPANDVARS, @Data);
end;


function ReportError(MsgID: integer; const AParam: PWideChar): integer; overload;

var
  Data: TErrorReport;
begin
  with Data do
  begin
    dwSize   := sizeof(TErrorReport);
    nMsgID   := MsgID;
    lpText   := AParam;
    nFlags   := 0;
  end;
  Result := TAPIProc(Adapter)(AU_PROC_REPORTERROR, @Data);
end;


function ReportError(const AParam: PWideChar): integer; overload;

begin
  Result := ReportError(AU_PROC_REPORTERROR, AParam);
end;


// Command returning value should be called WITHOUT first param, defining var name

// you will get the result in the AResult buffer instead

function CallCommand(const ACmdLine: PWideChar; AResult: PWideChar; AResultSize: DWORD): integer; overload;

var
  Data: TCallCmd;
begin
  with Data do
  begin
    lpText     := ACmdLine;
    lpBuffer   := AResult;
    dwBufSize  := AResultSize;
  end;
  Result := TAPIProc(Adapter)(AU_PROC_CALLCOMMAND, @Data);
end;


function CallCommand(const ACmdLine: PWideChar): integer; overload;

var
  Data: TCallCmd;
begin
  with Data do
  begin
    lpText     := ACmdLine;
    lpBuffer   := nil;
    dwBufSize  := 0;
  end;
  Result := TAPIProc(Adapter)(AU_PROC_CALLCOMMAND, @Data);
end;


function GetAutorunValue(AValueType: integer; const AValueName: PWideChar; ABuffer: Pointer; nBufSize, AFlags: DWORD): integer;

var

  Data: TValueInfo;
begin
  with Data do
  begin
    dwSize      := SizeOf(TValueInfo);
    dwValueType := AValueType;
    lpName      := AValueName;
    dwFlags     := AFlags;
    lpData      := ABuffer;
    nBufSize    := nBufSize;
  end;
  Result := TAPIProc(Adapter)(AU_PROC_GETVALUE, @Data);
end;



end.
